@@ -13,13 +13,13 @@
 import unittest
 import os
 import pathlib
-from elementpath import XPath1Parser, XPath2Parser, Selector, \
-    AttributeNode, TypedElement, ElementPathSyntaxError
+from xml.etree import ElementTree
 
-from xmlschema import XMLSchema10, XMLSchema11, XsdElement, XsdAttribute
+from elementpath import XPath1Parser, XPath2Parser, Selector, LazyElementNode
+
+from xmlschema import XMLSchema10, XMLSchema11
 from xmlschema.names import XSD_NAMESPACE
-from xmlschema.etree import ElementTree
-from xmlschema.xpath import XMLSchemaProxy, iter_schema_nodes, XPathElement
+from xmlschema.xpath import XMLSchemaProxy, XPathElement
 from xmlschema.validators import XsdAtomic, XsdAtomicRestriction
 
 CASES_DIR = os.path.join(os.path.dirname(__file__), 'test_cases/')
@@ -61,7 +61,7 @@ class XMLSchemaProxyTest(unittest.TestCase):
     def test_get_context_method(self):
         schema_proxy = XMLSchemaProxy(self.xs1)
         context = schema_proxy.get_context()
-        self.assertIs(context.root, self.xs1)
+        self.assertIs(context.root.value, self.xs1)
 
     def test_get_type_method(self):
         schema_proxy = XMLSchemaProxy(self.xs1)
@@ -118,40 +118,6 @@ class XMLSchemaProxyTest(unittest.TestCase):
             self.assertIsInstance(xsd_type, (XsdAtomic, XsdAtomicRestriction))
         self.assertGreater(k, 10)
 
-    def test_get_primitive_type_method(self):
-        schema_proxy = XMLSchemaProxy(self.xs3)
-
-        string_type = self.xs3.meta_schema.types['string']
-        xsd_type = self.xs3.types['list_of_strings']
-        self.assertIs(schema_proxy.get_primitive_type(xsd_type), string_type)
-
-        xsd_type = self.xs3.types['integer_or_float']
-        self.assertIs(schema_proxy.get_primitive_type(xsd_type), xsd_type)
-
-    def test_iter_schema_nodes_function(self):
-        vh_elements = set(e for e in self.xs1.maps.iter_components(XsdElement)
-                          if e.target_namespace == self.xs1.target_namespace)
-
-        self.assertEqual(set(iter_schema_nodes(self.xs1)), vh_elements | {self.xs1})
-        self.assertEqual(set(iter_schema_nodes(self.xs1, with_root=False)), vh_elements)
-
-        vh_nodes = set()
-        for node in self.xs1.maps.iter_components((XsdElement, XsdAttribute)):
-            if node.target_namespace != self.xs1.target_namespace:
-                continue
-            elif isinstance(node, XsdAttribute):
-                vh_nodes.add(AttributeNode(node.local_name, node))
-            else:
-                vh_nodes.add(node)
-
-        cars = self.xs1.elements['cars']
-        car = self.xs1.find('//vh:car')
-        typed_cars = TypedElement(cars, cars.type, None)
-        self.assertListEqual(list(iter_schema_nodes(cars)), [cars, car])
-        self.assertListEqual(list(iter_schema_nodes(typed_cars)), [cars, car])
-        self.assertListEqual(list(iter_schema_nodes(cars, with_root=False)), [car])
-        self.assertListEqual(list(iter_schema_nodes(typed_cars, with_root=False)), [car])
-
 
 class XPathElementTest(unittest.TestCase):
 
@@ -190,6 +156,13 @@ class XPathElementTest(unittest.TestCase):
         self.assertIsInstance(xpath_proxy, XMLSchemaProxy)
         self.assertIs(xpath_proxy._schema, self.col_schema)
 
+    def test_xpath_node(self):
+        elem = XPathElement('foo', self.col_schema.types['objType'])
+        xpath_node = elem.xpath_node
+        self.assertIsInstance(xpath_node, LazyElementNode)
+        self.assertIs(xpath_node, elem._xpath_node)
+        self.assertIs(xpath_node, elem.xpath_node)
+
     def test_schema(self):
         elem = XPathElement('foo', self.col_schema.types['objType'])
         self.assertIs(elem.schema, self.col_schema)
@@ -220,6 +193,7 @@ class XPathElementTest(unittest.TestCase):
 class XMLSchemaXPathTest(unittest.TestCase):
 
     schema_class = XMLSchema10
+    xs1: XMLSchema10
 
     @classmethod
     def setUpClass(cls):
@@ -229,10 +203,10 @@ class XMLSchemaXPathTest(unittest.TestCase):
         cls.bikes = cls.xs1.elements['vehicles'].type.content[1]
 
     def test_xpath_wrong_syntax(self):
-        self.assertRaises(ElementPathSyntaxError, self.xs1.find, './*[')
-        self.assertRaises(ElementPathSyntaxError, self.xs1.find, './*)')
-        self.assertRaises(ElementPathSyntaxError, self.xs1.find, './*3')
-        self.assertRaises(ElementPathSyntaxError, self.xs1.find, './@3')
+        self.assertRaises(SyntaxError, self.xs1.find, './*[')
+        self.assertRaises(SyntaxError, self.xs1.find, './*)')
+        self.assertRaises(SyntaxError, self.xs1.find, './*3')
+        self.assertRaises(SyntaxError, self.xs1.find, './@3')
 
     def test_xpath_extra_spaces(self):
         self.assertTrue(self.xs1.find('./ *') is not None)
@@ -273,6 +247,7 @@ class XMLSchemaXPathTest(unittest.TestCase):
 
     def test_xpath_predicate(self):
         car = self.xs1.elements['cars'].type.content[0]
+
         self.assertListEqual(self.xs1.findall("./vh:vehicles/vh:cars/vh:car[@make]"), [car])
         self.assertListEqual(self.xs1.findall("./vh:vehicles/vh:cars/vh:car[@make]"), [car])
         self.assertListEqual(self.xs1.findall("./vh:vehicles/vh:cars['ciao']"), [self.cars])
