@@ -352,20 +352,16 @@ class TestXMLSchema10(XsdValidatorTestCase):
             self.schema_class(self.vh_xsd_file, loglevel=logging.INFO)
 
         self.assertEqual(logger.level, logging.WARNING)
-        self.assertEqual(len(ctx.output), 7)
-        self.assertIn("INFO:xmlschema:Include schema from 'types.xsd'", ctx.output)
-        self.assertIn("INFO:xmlschema:Resource 'types.xsd' is already loaded", ctx.output)
+        self.assertEqual(len(ctx.output), 3)
+        self.assertIn("INFO:xmlschema:Include schema from ", ctx.output[0])
 
         with self.assertLogs('xmlschema', level='DEBUG') as ctx:
             self.schema_class(self.vh_xsd_file, loglevel=logging.DEBUG)
 
         self.assertEqual(logger.level, logging.WARNING)
-        self.assertEqual(len(ctx.output), 19)
-        self.assertIn("INFO:xmlschema:Include schema from 'cars.xsd'", ctx.output)
-        self.assertIn("INFO:xmlschema:Resource 'cars.xsd' is already loaded", ctx.output)
+        self.assertEqual(len(ctx.output), 38)
         self.assertIn("DEBUG:xmlschema:Schema targetNamespace is "
                       "'http://example.com/vehicles'", ctx.output)
-        self.assertIn("INFO:xmlschema:Resource 'cars.xsd' is already loaded", ctx.output)
 
         # With string argument
         with self.assertRaises(ValueError) as ctx:
@@ -374,11 +370,11 @@ class TestXMLSchema10(XsdValidatorTestCase):
 
         with self.assertLogs('xmlschema', level='INFO') as ctx:
             self.schema_class(self.vh_xsd_file, loglevel='INFO')
-        self.assertEqual(len(ctx.output), 7)
+        self.assertEqual(len(ctx.output), 3)
 
         with self.assertLogs('xmlschema', level='INFO') as ctx:
             self.schema_class(self.vh_xsd_file, loglevel='  Info ')
-        self.assertEqual(len(ctx.output), 7)
+        self.assertEqual(len(ctx.output), 3)
 
     def test_target_namespace(self):
         schema = self.schema_class(dedent("""\
@@ -700,9 +696,13 @@ class TestXMLSchema10(XsdValidatorTestCase):
         schema = CustomLocalXMLSchema(str(schema_file))
         self.assertTrue(schema.is_valid(str(xml_file)))
 
-        with self.assertRaises((pickle.PicklingError, AttributeError)) as ec:
+        with self.assertRaises((pickle.PicklingError, AttributeError)) as ec:  # type: ignore
             pickle.dumps(schema)
-        self.assertIn("Can't pickle", str(ec.exception))
+
+        error_message = str(ec.exception)
+        self.assertTrue(
+            "Can't get local object" in error_message or "Can't pickle" in error_message
+        )
 
     def test_meta_schema_validation(self):
         self.assertTrue(self.schema_class.meta_schema.is_valid(self.vh_xsd_file))
