@@ -9,14 +9,21 @@
 #
 from decimal import Decimal
 from math import isinf, isnan
-from typing import Optional, Set, Union
+from typing import Optional, Set, SupportsFloat, Union
 from xml.etree.ElementTree import Element
 from elementpath import datatypes
 
+from ..aliases import ElementType
+from ..names import XSD_ANNOTATION
 from ..exceptions import XMLSchemaValueError
+from ..translation import gettext as _
 from .exceptions import XMLSchemaValidationError
 
 XSD_FINAL_ATTRIBUTE_VALUES = {'restriction', 'extension', 'list', 'union'}
+XSD_BOOLEAN_MAP = {
+    'false': False, '0': False,
+    'true': True, '1': True
+}
 
 
 def get_xsd_derivation_attribute(elem: Element, attribute: str,
@@ -40,10 +47,24 @@ def get_xsd_derivation_attribute(elem: Element, attribute: str,
     if len(items) == 1 and items[0] == '#all':
         return ' '.join(values)
     elif not all(s in values for s in items):
-        raise ValueError("wrong value %r for attribute %r" % (value, attribute))
+        raise ValueError(_("wrong value %r for attribute %r") % (value, attribute))
     return value
 
 
+def get_xsd_annotation_child(elem: ElementType) -> Optional[ElementType]:
+    """
+    Returns the child element of the annotation associated to an XSD component,
+    `None` if it doesn't exist.
+    """
+    for child in elem:
+        if child.tag == XSD_ANNOTATION:
+            return child
+        elif not callable(child.tag):
+            return None
+    else:
+        return None
+
+
 #
 # XSD built-in types validator functions
 
@@ -55,92 +76,92 @@ def decimal_validator(value: Union[Decimal, int, float, str]) -> None:
             raise ValueError()
     except (ValueError, TypeError):
         raise XMLSchemaValidationError(decimal_validator, value,
-                                       "value is not a valid xs:decimal") from None
+                                       _("value is not a valid xs:decimal")) from None
 
 
 def qname_validator(value: str) -> None:
     if datatypes.QName.pattern.match(value) is None:
         raise XMLSchemaValidationError(qname_validator, value,
-                                       "value is not an xs:QName")
+                                       _("value is not an xs:QName"))
 
 
 def byte_validator(value: int) -> None:
     if not (-2**7 <= value < 2 ** 7):
         raise XMLSchemaValidationError(int_validator, value,
-                                       "value must be -128 <= x < 128")
+                                       _("value must be {:s}").format("-128 <= x < 128"))
 
 
 def short_validator(value: int) -> None:
     if not (-2**15 <= value < 2 ** 15):
         raise XMLSchemaValidationError(short_validator, value,
-                                       "value must be -2^15 <= x < 2^15")
+                                       _("value must be {:s}").format("-2^15 <= x < 2^15"))
 
 
 def int_validator(value: int) -> None:
     if not (-2**31 <= value < 2 ** 31):
         raise XMLSchemaValidationError(int_validator, value,
-                                       "value must be -2^31 <= x < 2^31")
+                                       _("value must be {:s}").format("-2^31 <= x < 2^31"))
 
 
 def long_validator(value: int) -> None:
     if not (-2**63 <= value < 2 ** 63):
         raise XMLSchemaValidationError(long_validator, value,
-                                       "value must be -2^63 <= x < 2^63")
+                                       _("value must be {:s}").format("-2^63 <= x < 2^63"))
 
 
 def unsigned_byte_validator(value: int) -> None:
     if not (0 <= value < 2 ** 8):
         raise XMLSchemaValidationError(unsigned_byte_validator, value,
-                                       "value must be 0 <= x < 256")
+                                       _("value must be {:s}").format("0 <= x < 256"))
 
 
 def unsigned_short_validator(value: int) -> None:
     if not (0 <= value < 2 ** 16):
         raise XMLSchemaValidationError(unsigned_short_validator, value,
-                                       "value must be 0 <= x < 2^16")
+                                       _("value must be {:s}").format("0 <= x < 2^16"))
 
 
 def unsigned_int_validator(value: int) -> None:
     if not (0 <= value < 2 ** 32):
         raise XMLSchemaValidationError(unsigned_int_validator, value,
-                                       "value must be 0 <= x < 2^32")
+                                       _("value must be {:s}").format("0 <= x < 2^32"))
 
 
 def unsigned_long_validator(value: int) -> None:
     if not (0 <= value < 2 ** 64):
         raise XMLSchemaValidationError(unsigned_long_validator, value,
-                                       "value must be 0 <= x < 2^64")
+                                       _("value must be {:s}").format("0 <= x < 2^64"))
 
 
 def negative_int_validator(value: int) -> None:
     if value >= 0:
         raise XMLSchemaValidationError(negative_int_validator, value,
-                                       "value must be negative")
+                                       _("value must be negative"))
 
 
 def positive_int_validator(value: int) -> None:
     if value <= 0:
         raise XMLSchemaValidationError(positive_int_validator, value,
-                                       "value must be positive")
+                                       _("value must be positive"))
 
 
 def non_positive_int_validator(value: int) -> None:
     if value > 0:
         raise XMLSchemaValidationError(non_positive_int_validator, value,
-                                       "value must be non positive")
+                                       _("value must be non positive"))
 
 
 def non_negative_int_validator(value: int) -> None:
     if value < 0:
         raise XMLSchemaValidationError(non_negative_int_validator, value,
-                                       "value must be non negative")
+                                       _("value must be non negative"))
 
 
 def hex_binary_validator(value: Union[str, datatypes.HexBinary]) -> None:
     if not isinstance(value, datatypes.HexBinary) and \
             datatypes.HexBinary.pattern.match(value) is None:
         raise XMLSchemaValidationError(hex_binary_validator, value,
-                                       "not an hexadecimal number")
+                                       _("not an hexadecimal number"))
 
 
 def base64_binary_validator(value: Union[str, datatypes.Base64Binary]) -> None:
@@ -153,25 +174,33 @@ def base64_binary_validator(value: Union[str, datatypes.Base64Binary]) -> None:
     match = datatypes.Base64Binary.pattern.match(value)
     if match is None or match.group(0) != value:
         raise XMLSchemaValidationError(base64_binary_validator, value,
-                                       "not a base64 encoding")
+                                       _("not a base64 encoding"))
 
 
 def error_type_validator(value: object) -> None:
     raise XMLSchemaValidationError(error_type_validator, value,
-                                   "no value is allowed for xs:error type")
+                                   _("no value is allowed for xs:error type"))
 
 
 #
 # XSD builtin decoding functions
 
 def boolean_to_python(value: str) -> bool:
-    if value in {'true', '1'}:
-        return True
-    elif value in {'false', '0'}:
-        return False
-    else:
-        raise XMLSchemaValueError('{!r} is not a boolean value'.format(value))
+    try:
+        return XSD_BOOLEAN_MAP[value]
+    except KeyError:
+        raise XMLSchemaValueError(_('{!r} is not a boolean value').format(value))
 
 
 def python_to_boolean(value: object) -> str:
     return str(value).lower()
+
+
+def python_to_float(value: SupportsFloat) -> str:
+    if isnan(value):
+        return "NaN"
+    if value == float("inf"):
+        return "INF"
+    if value == float("-inf"):
+        return "-INF"
+    return str(value)
